#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import logging
import syslog
import typing
import unittest
from datetime import timedelta
from unittest import mock

import daiquiri
from daiquiri import output


class DatadogMatcher(object):
    def __init__(self, expected: typing.Any) -> None:
        self.expected = expected

    def __eq__(self, other: typing.Any) -> bool:
        return bool(json.loads(other.decode()[:-1]) == self.expected)

    def __repr__(self) -> str:
        return (
            "b'"
            + json.dumps(self.expected, default=lambda x: "unserializable")
            + "\\n'"
        )


class TestOutput(unittest.TestCase):
    def test_find_facility(self) -> None:
        self.assertEqual(syslog.LOG_USER, output.Syslog._find_facility("user"))
        self.assertEqual(syslog.LOG_LOCAL1, output.Syslog._find_facility("log_local1"))
        self.assertEqual(syslog.LOG_LOCAL2, output.Syslog._find_facility("LOG_local2"))
        self.assertEqual(syslog.LOG_LOCAL3, output.Syslog._find_facility("LOG_LOCAL3"))
        self.assertEqual(syslog.LOG_LOCAL4, output.Syslog._find_facility("LOCaL4"))

    def test_get_log_file_path(self) -> None:
        self.assertEqual("foobar.log", output._get_log_file_path("foobar.log"))
        self.assertEqual(
            "/var/log/foo/foobar.log",
            output._get_log_file_path("foobar.log", logdir="/var/log/foo"),
        )
        self.assertEqual(
            "/var/log/foobar.log",
            output._get_log_file_path(logdir="/var/log", program_name="foobar"),
        )
        self.assertEqual(
            "/var/log/foobar.log",
            output._get_log_file_path(logdir="/var/log", program_name="foobar"),
        )
        self.assertEqual(
            "/var/log/foobar.journal",
            output._get_log_file_path(
                logdir="/var/log", logfile_suffix=".journal", program_name="foobar"
            ),
        )

    def test_timedelta_seconds(self) -> None:
        fn = output.TimedRotatingFile._timedelta_to_seconds
        hour = 60 * 60  # seconds * minutes

        one_hour = [
            timedelta(hours=1),
            timedelta(minutes=60),
            timedelta(seconds=hour),
        ]
        for t in one_hour:
            self.assertEqual(hour, fn(t))

        assert hour == fn(float(hour))
        assert hour == fn(hour)

        error_cases = [
            "string",
            ["some", "list"],
            (
                "some",
                "tuple",
            ),
            ("tuple",),
            {"dict": "mapping"},
        ]
        for t in error_cases:  # type: ignore[assignment]
            self.assertRaises(AttributeError, fn, t)

    def test_datadog(self) -> None:
        with mock.patch("socket.socket") as mock_socket:
            socket_instance = mock_socket.return_value
            daiquiri.setup(outputs=(daiquiri.output.Datadog(),), level=logging.DEBUG)
            logger = daiquiri.getLogger()
            logger.error("foo", bar=1)
            logger.info("bar")
            expected_error_1 = {
                "status": "error",
                "message": "foo",
                "bar": 1,
                "logger": {"name": "root"},
                "timestamp": mock.ANY,
            }
            expected_info_1 = {
                "status": "info",
                "message": "bar",
                "logger": {"name": "root"},
                "timestamp": mock.ANY,
            }
            expected_error_2 = {
                "status": "error",
                "message": "backtrace",
                "logger": {"name": "saymyname"},
                "timestamp": mock.ANY,
                "error": {
                    "kind": "ZeroDivisionError",
                    "stack": None,
                    "message": mock.ANY,
                },
            }
            try:
                1 / 0
            except ZeroDivisionError:
                logger = daiquiri.getLogger("saymyname")
                logger.exception("backtrace")
            socket_instance.connect.assert_called_once_with(("127.0.0.1", 10518))
            socket_instance.sendall.assert_has_calls(
                [
                    mock.call(DatadogMatcher(expected_error_1)),
                    mock.call(DatadogMatcher(expected_info_1)),
                    mock.call(DatadogMatcher(expected_error_2)),
                ]
            )



import logging

import daiquiri
import daiquiri.formatter

daiquiri.setup(
    level=logging.INFO,
    outputs=[
        daiquiri.output.Stream(
            formatter=daiquiri.formatter.ColorExtrasFormatter(
                fmt=(
                    daiquiri.formatter.DEFAULT_FORMAT
                    + " [%(subsystem)s is %(mood)s]"
                    + "%(extras)s"
                ),
                keywords=["mood", "subsystem"],
            )
        )
    ],
)

logger = daiquiri.getLogger(__name__, subsystem="example")
logger.info(
    "It works and log to stderr by default with color!",
    mood="happy",
    arbitrary_context="included",
)



import logging

import daiquiri
import daiquiri.formatter

daiquiri.setup(
    level=logging.INFO,
    outputs=(
        daiquiri.output.Stream(
            formatter=daiquiri.formatter.ColorFormatter(
                fmt="%(asctime)s [PID %(process)d] [%(levelname)s] "
                "%(name)s -> %(message)s"
            )
        ),
    ),
)

logger = daiquiri.getLogger(__name__)
logger.info("It works with a custom format!")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import io
import logging
import unittest

import daiquiri


class TestColorExtrasFormatter(unittest.TestCase):
    logger: daiquiri.KeywordArgumentAdapter
    stream: io.StringIO
    handler: daiquiri.handlers.TTYDetectorStreamHandler

    @classmethod
    def setUpClass(cls) -> None:
        cls.logger = daiquiri.getLogger("my_module")
        cls.logger.setLevel(logging.INFO)
        cls.stream = io.StringIO()
        cls.handler = daiquiri.handlers.TTYDetectorStreamHandler(cls.stream)
        cls.logger.logger.addHandler(cls.handler)
        super(TestColorExtrasFormatter, cls).setUpClass()

    def setUp(self) -> None:
        # Couldn't get readline() to return anything no matter what I tried, so
        # getvalue() is the only way to see what's in the stream. However this
        # requires the stream to be reset every time.
        self.stream.close()
        self.stream = io.StringIO()
        self.handler.stream = self.stream
        super(TestColorExtrasFormatter, self).setUp()

    def test_no_keywords(self) -> None:
        format_string = "%(levelname)s %(name)s%(extras)s: %(message)s"
        formatter = daiquiri.formatter.ColorExtrasFormatter(fmt=format_string)
        self.handler.setFormatter(formatter)

        self.logger.info("test message")
        self.assertEqual(self.stream.getvalue(), "INFO my_module: test message\n")

    def test_no_keywords_with_extras(self) -> None:
        format_string = "%(levelname)s %(name)s%(extras)s: %(message)s"
        formatter = daiquiri.formatter.ColorExtrasFormatter(fmt=format_string)
        self.handler.setFormatter(formatter)

        self.logger.info("test message", test="a")
        self.assertEqual(
            self.stream.getvalue(), "INFO my_module [test: a]: test message\n"
        )

    def test_empty_keywords(self) -> None:
        format_string = "%(levelname)s %(name)s%(extras)s: %(message)s"
        formatter = daiquiri.formatter.ColorExtrasFormatter(
            fmt=format_string,
            keywords=set(),
        )
        self.handler.setFormatter(formatter)

        self.logger.info("test message", test="a")
        self.assertEqual(
            self.stream.getvalue(), "INFO my_module [test: a]: test message\n"
        )

    def test_keywords_no_extras(self) -> None:
        format_string = "%(levelname)s %(name)s" " %(test)s%(extras)s: %(message)s"
        formatter = daiquiri.formatter.ColorExtrasFormatter(
            fmt=format_string, keywords={"test"}
        )
        self.handler.setFormatter(formatter)

        self.logger.info("test message", test="a")
        self.assertEqual(self.stream.getvalue(), "INFO my_module a: test message\n")

    def test_keywords_with_extras(self) -> None:
        format_string = "%(levelname)s %(name)s" " %(test)s%(extras)s: %(message)s"
        formatter = daiquiri.formatter.ColorExtrasFormatter(
            fmt=format_string, keywords={"test"}
        )
        self.handler.setFormatter(formatter)

        self.logger.info("test message", test="a", test2="b")
        self.assertEqual(
            self.stream.getvalue(), "INFO my_module a [test2: b]: test message\n"
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Formatters."""

import logging
import typing

from pythonjsonlogger import json as jsonlogger

from daiquiri import types


DEFAULT_FORMAT = (
    "%(asctime)s [%(process)d] %(color)s%(levelname)-8.8s "
    "%(name)s: %(message)s%(color_stop)s"
)

DEFAULT_EXTRAS_FORMAT = (
    "%(asctime)s [%(process)d] %(color)s%(levelname)-8.8s "
    "%(name)s%(extras)s: %(message)s%(color_stop)s"
)


class ColorFormatter(logging.Formatter):
    """Colorizes log output."""

    # TODO(jd) Allow configuration
    LEVEL_COLORS = {
        logging.DEBUG: "\033[00;32m",  # GREEN
        logging.INFO: "\033[00;36m",  # CYAN
        logging.WARN: "\033[01;33m",  # BOLD YELLOW
        logging.ERROR: "\033[01;31m",  # BOLD RED
        logging.CRITICAL: "\033[01;31m",  # BOLD RED
    }

    COLOR_STOP = "\033[0m"

    def add_color(self, record: types.ColoredLogRecord) -> None:
        """Add color to a record."""
        if getattr(record, "_stream_is_a_tty", False):
            record.color = self.LEVEL_COLORS[record.levelno]
            record.color_stop = self.COLOR_STOP
        else:
            record.color = ""
            record.color_stop = ""

    def remove_color(self, record: types.ColoredLogRecord) -> None:
        """Remove color from a record."""
        del record.color
        del record.color_stop

    def format(self, record: logging.LogRecord) -> str:
        """Format a record."""
        record = typing.cast(types.ColoredLogRecord, record)
        self.add_color(record)
        s = super(ColorFormatter, self).format(record)
        self.remove_color(record)
        return s


class ExtrasFormatter(logging.Formatter):
    """Formats extra keywords into %(extras)s placeholder.

    Any keywords passed to a logging call will be formatted into a
    "extras" string and included in a logging message.

    Example:
        logger.info('my message', extra='keyword')

    will cause an "extras" string of:
        [extra: keyword]

    to be inserted into the format in place of %(extras)s.

    The optional `keywords` argument must be passed into the init
    function to enable this functionality. Without it normal daiquiri
    formatting will be applied. Any keywords included in the
    `keywords` parameter will not be included in the "extras" string.

    Special keywords:

    keywords
        A set of strings containing keywords to filter out of the
        "extras" string.

    extras_template
        A format string to use instead of '[{0}: {1}]'

    extras_separator
        What string to "join" multiple "extras" with.

    extras_prefix and extras_suffix
        Strings which will be prepended and appended to the "extras"
        string respectively. These will only be prepended if the
        "extras" string is not empty.
    """

    def __init__(
        self,
        keywords: typing.Optional[typing.Set[str]] = None,
        extras_template: str = "[{0}: {1}]",
        extras_separator: str = " ",
        extras_prefix: str = " ",
        extras_suffix: str = "",
        *args: typing.Any,
        **kwargs: typing.Any,
    ) -> None:
        self.keywords = set() if keywords is None else keywords
        self.extras_template = extras_template
        self.extras_separator = extras_separator
        self.extras_prefix = extras_prefix
        self.extras_suffix = extras_suffix
        super(ExtrasFormatter, self).__init__(*args, **kwargs)

    def add_extras(self, record: types.ExtrasLogRecord) -> None:
        if not hasattr(record, "_daiquiri_extra_keys"):
            record.extras = ""
            return

        extras = self.extras_separator.join(
            self.extras_template.format(k, getattr(record, k))
            for k in record._daiquiri_extra_keys
            if k != "_daiquiri_extra_keys" and k not in self.keywords
        )
        if extras != "":
            extras = self.extras_prefix + extras + self.extras_suffix
        record.extras = extras

    def remove_extras(self, record: types.ExtrasLogRecord) -> None:
        del record.extras

    def format(self, record: logging.LogRecord) -> str:
        record = typing.cast(types.ExtrasLogRecord, record)
        self.add_extras(record)
        s = super(ExtrasFormatter, self).format(record)
        self.remove_extras(record)
        return s


class ColorExtrasFormatter(ColorFormatter, ExtrasFormatter):
    """Combines functionality of ColorFormatter and ExtrasFormatter."""

    def format(self, record: logging.LogRecord) -> str:
        record = typing.cast(types.ColoredLogRecord, record)
        self.add_color(record)
        s = ExtrasFormatter.format(self, record)
        self.remove_color(record)
        return s


class DatadogFormatter(jsonlogger.JsonFormatter):
    def __init__(self) -> None:
        super(DatadogFormatter, self).__init__(timestamp=True)

    def add_fields(
        self,
        log_record: typing.Dict[str, typing.Any],
        record: logging.LogRecord,
        message_dict: typing.Dict[str, str],
    ) -> None:
        super(DatadogFormatter, self).add_fields(log_record, record, message_dict)
        log_record["status"] = record.levelname.lower()
        log_record["logger"] = {
            "name": record.name,
        }
        if record.exc_info and record.exc_info[0]:
            log_record["error"] = {
                "kind": record.exc_info[0].__name__,
                "stack": message_dict.get("stack_info"),
                "message": message_dict.get("exc_info"),
            }
            log_record.pop("exc_info", None)
            log_record.pop("stack_info", None)


TEXT_FORMATTER = ColorExtrasFormatter(fmt=DEFAULT_EXTRAS_FORMAT)
JSON_FORMATTER = jsonlogger.JsonFormatter()
DATADOG_FORMATTER = DatadogFormatter()



import logging

import daiquiri
import daiquiri.formatter

daiquiri.setup(
    level=logging.INFO,
    outputs=(
        daiquiri.output.Stream(
            formatter=daiquiri.formatter.ColorFormatter(
                fmt=(daiquiri.formatter.DEFAULT_FORMAT + " [%(subsystem)s is %(mood)s]")
            )
        ),
    ),
)

logger = daiquiri.getLogger(__name__, subsystem="example")
logger.info("It works and log to stderr by default with color!", mood="happy")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import datetime
import inspect
import logging
import logging.handlers
import os
import sys
import typing

try:
    import syslog
except ImportError:
    syslog = None  # type: ignore[assignment]

from daiquiri import formatter
from daiquiri import handlers


def get_program_name() -> str:
    """Return the name of the running program."""
    return os.path.basename(inspect.stack()[-1][1])


class Output:
    """Generic log output."""

    def __init__(
        self,
        handler: logging.Handler,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
    ):
        self.handler = handler
        self.handler.setFormatter(formatter)
        if level is not None:
            self.handler.setLevel(level)

    def add_to_logger(self, logger: logging.Logger) -> None:
        """Add this output to a logger."""
        logger.addHandler(self.handler)


def _get_log_file_path(
    logfile: typing.Optional[str] = None,
    logdir: typing.Optional[str] = None,
    program_name: typing.Optional[str] = None,
    logfile_suffix: str = ".log",
) -> str:
    ret_path = None

    if not logdir:
        ret_path = logfile

    if not ret_path and logfile and logdir:
        ret_path = os.path.join(logdir, logfile)

    if not ret_path and logdir:
        program_name = program_name or get_program_name()
        ret_path = os.path.join(logdir, program_name) + logfile_suffix

    if not ret_path:
        raise ValueError("Unable to determine log file destination")

    return ret_path


class File(Output):
    """Ouput to a file."""

    def __init__(
        self,
        filename: typing.Optional[str] = None,
        directory: typing.Optional[str] = None,
        suffix: str = ".log",
        program_name: typing.Optional[str] = None,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
    ):
        """Log file output.

        :param filename: The log file path to write to. If directory is also
                         specified, both will be combined.

        :param directory: The log directory to write to. If no filename is
                          specified, the program name and suffix will be used
                          to contruct the full path relative to the directory.

        :param suffix: The log file name suffix. This will be only used if no
                       filename has been provided.

        :param program_name: Program name. Autodetected by default.

        """
        logpath = _get_log_file_path(filename, directory, program_name, suffix)
        handler = logging.handlers.WatchedFileHandler(logpath)
        super(File, self).__init__(handler, formatter, level)


class RotatingFile(Output):
    """Output to a file, rotating after a certain size."""

    handler: logging.handlers.RotatingFileHandler

    def __init__(
        self,
        filename: typing.Optional[str] = None,
        directory: typing.Optional[str] = None,
        suffix: str = ".log",
        program_name: typing.Optional[str] = None,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
        max_size_bytes: int = 0,
        backup_count: int = 0,
    ):
        """Rotating log file output.

        :param filename: The log file path to write to. If directory is also
                         specified, both will be combined.

        :param directory: The log directory to write to. If no filename is
                          specified, the program name and suffix will be used
                          to contruct the full path relative to the directory.

        :param suffix: The log file name suffix. This will be only used if no
                       filename has been provided.

        :param program_name: Program name. Autodetected by default.

        :param max_size_bytes: Allow the file to rollover at a predetermined
                               size.

        :param backup_count: The maximum number of files to rotate logging
                             output between.

        """
        logpath = _get_log_file_path(filename, directory, program_name, suffix)
        handler = logging.handlers.RotatingFileHandler(
            logpath, maxBytes=max_size_bytes, backupCount=backup_count
        )
        super(RotatingFile, self).__init__(handler, formatter, level)

    def do_rollover(self) -> None:
        """Manually forces a log file rotation."""
        return self.handler.doRollover()


class TimedRotatingFile(Output):
    """Rotating log file output, triggered by a fixed interval."""

    handler: logging.handlers.TimedRotatingFileHandler

    def __init__(
        self,
        filename: typing.Optional[str] = None,
        directory: typing.Optional[str] = None,
        suffix: str = ".log",
        program_name: typing.Optional[str] = None,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
        interval: typing.Union[float, int, datetime.timedelta] = datetime.timedelta(
            hours=24
        ),
        backup_count: int = 0,
    ):
        """Rotating log file output, triggered by a fixed interval.

        :param filename: The log file path to write to. If directory is also
                         specified, both will be combined.

        :param directory: The log directory to write to. If no filename is
                          specified, the program name and suffix will be used
                          to contruct the full path relative to the directory.

        :param suffix: The log file name suffix. This will be only used if no
                       filename has been provided.

        :param program_name: Program name. Autodetected by default.

        :param interval: datetime.timedelta instance representing how often a
                         new log file should be created.

        :param backup_count: The maximum number of files to rotate logging
                             output between.
        """
        logpath = _get_log_file_path(filename, directory, program_name, suffix)
        handler = logging.handlers.TimedRotatingFileHandler(
            logpath,
            when="S",
            interval=int(self._timedelta_to_seconds(interval)),
            backupCount=backup_count,
        )
        super(TimedRotatingFile, self).__init__(handler, formatter, level)

    def do_rollover(self) -> None:
        """Manually forces a log file rotation."""
        return self.handler.doRollover()

    @staticmethod
    def _timedelta_to_seconds(
        td: typing.Union[float, int, datetime.timedelta]
    ) -> float:
        """Convert a datetime.timedelta object into a seconds interval.

        :param td: datetime.timedelta
        :return: time in seconds
        :rtype: int
        """
        if isinstance(td, (int, float)):
            td = datetime.timedelta(seconds=td)
        return td.total_seconds()


class Stream(Output):
    """Generic stream output."""

    def __init__(
        self,
        stream: typing.TextIO = sys.stderr,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
    ) -> None:
        super(Stream, self).__init__(
            handlers.TTYDetectorStreamHandler(stream), formatter, level
        )


STDERR = Stream()
STDOUT = Stream(sys.stdout)


class Journal(Output):
    def __init__(
        self,
        program_name: typing.Optional[str] = None,
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
    ) -> None:
        program_name = program_name or get_program_name()
        super(Journal, self).__init__(
            handlers.JournalHandler(program_name), formatter, level
        )


class Syslog(Output):
    def __init__(
        self,
        program_name: typing.Optional[str] = None,
        facility: str = "user",
        formatter: logging.Formatter = formatter.TEXT_FORMATTER,
        level: typing.Optional[int] = None,
    ) -> None:
        if syslog is None:
            # FIXME(jd) raise something more specific
            raise RuntimeError("syslog is not available on this platform")
        super(Syslog, self).__init__(
            handlers.SyslogHandler(
                program_name=program_name or get_program_name(),
                facility=self._find_facility(facility),
            ),
            formatter,
            level,
        )

    @staticmethod
    def _find_facility(facility: str) -> int:
        # NOTE(jd): Check the validity of facilities at run time as they differ
        # depending on the OS and Python version being used.
        valid_facilities = [
            f
            for f in [
                "LOG_KERN",
                "LOG_USER",
                "LOG_MAIL",
                "LOG_DAEMON",
                "LOG_AUTH",
                "LOG_SYSLOG",
                "LOG_LPR",
                "LOG_NEWS",
                "LOG_UUCP",
                "LOG_CRON",
                "LOG_AUTHPRIV",
                "LOG_FTP",
                "LOG_LOCAL0",
                "LOG_LOCAL1",
                "LOG_LOCAL2",
                "LOG_LOCAL3",
                "LOG_LOCAL4",
                "LOG_LOCAL5",
                "LOG_LOCAL6",
                "LOG_LOCAL7",
            ]
            if getattr(syslog, f, None)
        ]

        facility = facility.upper()

        if not facility.startswith("LOG_"):
            facility = "LOG_" + facility

        if facility not in valid_facilities:
            raise TypeError(
                "syslog facility must be one of: %s"
                % ", ".join("'%s'" % fac for fac in valid_facilities)
            )

        return int(getattr(syslog, facility))


class Datadog(Output):
    def __init__(
        self,
        hostname: str = "127.0.0.1",
        port: int = 10518,
        formatter: logging.Formatter = formatter.DATADOG_FORMATTER,
        level: typing.Optional[int] = None,
        handler_class: typing.Type[
            logging.handlers.SocketHandler
        ] = handlers.PlainTextSocketHandler,
    ):
        super(Datadog, self).__init__(
            handler_class(hostname, port),
            formatter=formatter,
            level=level,
        )


# FIXME(jd): Is this useful? Remove it?
preconfigured: typing.Dict[str, typing.Union[Stream, Output]] = {
    "stderr": STDERR,
    "stdout": STDOUT,
}

if syslog is not None:
    preconfigured["syslog"] = Syslog()

if handlers.journal is not None:  # type: ignore[attr-defined]
    preconfigured["journal"] = Journal()



import datetime
import logging

import daiquiri

daiquiri.setup(
    level=logging.DEBUG,
    outputs=(
        daiquiri.output.File("errors.log", level=logging.ERROR),
        daiquiri.output.TimedRotatingFile(
            "everything.log", level=logging.DEBUG, interval=datetime.timedelta(hours=1)
        ),
    ),
)

logger = daiquiri.getLogger(__name__)

logger.info("only to rotating file logger")
logger.error("both log files, including errors only")



import logging
import sys

import daiquiri

# Log both to stdout and as JSON in a file called /dev/null. (Requires
# `python-json-logger`)
daiquiri.setup(
    level=logging.INFO,
    outputs=(
        daiquiri.output.Stream(sys.stdout),
        daiquiri.output.File("/dev/null", formatter=daiquiri.formatter.JSON_FORMATTER),
    ),
)

logger = daiquiri.getLogger(__name__, subsystem="example")
logger.info("It works and log to stdout and /dev/null with JSON")
